/*
 * BC7215 Library Implementation
 * 
 * Description:
 *   This source file contains the implementation of the BC7215 library. It provides 
 *   the functionality to interact with the BC7215 chip, including setting up the 
 *   hardware interface, managing data transmission and reception, and controlling 
 *   different operational modes. This implementation relies on the definitions and 
 *   configurations set in 'bc7215_lib.h' and 'bc7215_lib_config.h'.
 *
 * Author: Bitcode
 * Date: 2024-01-12
 * 
 * Version: 1.0
 * 
 * Notes:
 *   The functions defined in this file are designed to provide a seamless interface 
 *   for working with the BC7215 IR decoder/encoder chip. Users should familiarize 
 *   themselves with the API provided and configure the library according to their 
 *   application's requirements using 'bc7215_lib_config.h'.
 * 
 *   The implementation assumes a compatible hardware setup and correct configuration 
 *   settings. It is crucial for users to ensure that their hardware setup matches 
 *   the expectations of this library to avoid any operational issues.
 *
 */

#include "bc7215_lib.h"


// **********************************************************
// *** Callback function pointers for the BC7215 library. ***
// **********************************************************
// These pointers point to hardware related functions.
// All these pointers must be configured before any other library functions can be called.
voidFunction_t			enUartIntFunction = NULL;		// pointer to UART interrupt disable function
voidFunction_t			disUartIntFunction = NULL;		// pointer to UART interrupt enable function
uint8ParamFunction_t	uartSendByteFunction = NULL;	// pointer to UART send 1 byte function
voidFunction_t			modSetHighFunction = NULL;		// pointer to mod pin set high function
voidFunction_t			modSetLowFunction = NULL;		// pointer to mod pin set low function
readIOFunction_t 		readModFunction = NULL;			// pointer to read mod pin function
readIOFunction_t 		readBusyFunction = NULL;		// pointer to read busy pin function


// ****************************************************************
// *** Internal variables and functions for the BC7215 library. ***
// ****************************************************************
struct {
	uint8_t		formatPktReady : 1;
	uint8_t		dataPktReady : 1;
	uint8_t		newPkt : 1;
	uint8_t		overLap : 1;
	uint8_t		cmdComplete : 1;
} bc7215Status;

#if (BC7215_MAX_DATA_BYTE_LENGTH+2)+1 > (32+1)
// (BC7215_MAX_DATA_BYTE_LENGTH+2)+1 is the size of whole data packet
// (32+1) is the size of format packet

	// if maximum data packet length is larger than format packet
	#define	BC7215_BUFFER_SIZE	(2*((BC7215_MAX_DATA_BYTE_LENGTH+2)+1))

#else

	// if if maximum data packet length is smaller or equal to format packet
	#define	BC7215_BUFFER_SIZE	 ((BC7215_MAX_DATA_BYTE_LENGTH+2)+1+(32+1))

#endif

uint8_t		circularBuffer[BC7215_BUFFER_SIZE];
uint16_t	bitLen;

#if BC7215_BUFFER_SIZE < 256
	uint8_t		startPos;
	uint8_t		datStartPos;
	uint8_t		lastWritingPos;
	uint8_t		datEndPos;
	uint8_t		byteCount;
	uint8_t		datCount;
	static uint8_t 	bufBackRead(uint8_t pos, uint8_t n);
	static uint8_t 	bufRead(uint8_t pos, uint8_t n);
#else
	uint16_t	startPos;
	uint16_t	datStartPos;
	uint16_t	lastWritingPos;
	uint16_t	datEndPos;
	uint16_t	byteCount;
	uint16_t	datCount;
	static uint8_t 	bufBackRead(uint16_t pos, uint16_t n);
	static uint8_t 	bufRead(uint16_t pos, uint16_t n);
#endif
// bufBackRead(): read the (n)th byte backwards from the pos, n range from 0 to BC7215_BUFFER_SIZE-1
// bufRead():  read the (n)th byte after the pos, n range from 0 to BC7215_BUFFER_SIZE-1

static void byte_stuffing_send(uint8_t data);


// ****************************************
// *** Callback configuration functions ***
// ****************************************

// UART interrupt enable function configuration
// The function must have a signature of " void function_name(void) "
void bc7215_config_uart_int_en_funct(voidFunction_t en_int_function)
{
	enUartIntFunction = en_int_function;
	if (enUartIntFunction == NULL)		// dead trap for misconfiguration
	{
		while(1);
	}
}

// UART interrupt disable function configuration
// The function must have a signature of " void function_name(void) "
void bc7215_config_uart_int_dis_funct(voidFunction_t dis_int_function)
{
	disUartIntFunction = dis_int_function;
	if (disUartIntFunction == NULL)		// dead trap for misconfiguration
	{
		while(1);
	}
}

// UART send 1 byte function configuration
// The function must have a signature of " void function_name(uint8_t) ",
// and this function should only return after the data has been completely sent.
void bc7215_config_uart_send_byte(uint8ParamFunction_t send_byte_function)
{
	uartSendByteFunction = send_byte_function;
	if (uartSendByteFunction == NULL)		// dead trap for misconfiguration
	{
		while(1);
	}
}

// MOD pin set high function configuration
// The function must have a signature of " void function_name(void) ",
void bc7215_config_mod_set_high_funct(voidFunction_t set_mod_function)
{
	modSetHighFunction = set_mod_function;
	if (modSetHighFunction == NULL)		// dead trap for misconfiguration
	{
		while(1);
	}
}

// MOD pin set low function configuration
// The function must have a signature of " void function_name(void) ",
void bc7215_config_mod_set_low_funct(voidFunction_t clear_mod_function)
{
	modSetLowFunction = clear_mod_function;
	if (modSetLowFunction == NULL)		// dead trap for misconfiguration
	{
		while(1);
	}
}

// Read MOD pin function configuration
// The function must have a signature of " uint8_t function_name(void) ",
// return 0 if MOD pin is low, non-zero if MOD pin is high
void bc7215_config_read_mod_funct(readIOFunction_t read_mod_function)
{
	readModFunction = read_mod_function;
	if (readModFunction == NULL)		// dead trap for misconfiguration
	{
		while(1);
	}
}

// Read BUSY pin function configuration
// The function must have a signature of " uint8_t function_name(void) ",
// return 0 if BUSY pin is low, non-zero if BUSY pin is high
void bc7215_config_read_busy_funct(readIOFunction_t read_busy_function)
{
	readBusyFunction = read_busy_function;
	if (readBusyFunction == NULL)		// dead trap for misconfiguration
	{
		while(1);
	}
}

// *****************************
// *** data process function ***
// *****************************
// this function should be called from the user UART RX interrupt service routine when a byte
// is received from BC7215.
void bc7215_process_uart_data(uint8_t data)
{
static uint8_t	previousData=0;
uint8_t		temp;
uint16_t	temp16;
	if (readModFunction() == 0)			// MOD=0 means bc7215 is in transmit mode
	{
		if (data == 0x7a)
		{
			bc7215Status.cmdComplete = 1;
		}
	}
	else		// bc7215 is in receive mode
	{
		if (data == 0x7a)
		{
			if (previousData == 0x7a)		// if 0x7a 0x7a is the mark of format packet
			{
				if (byteCount == sizeof(bc7215FormatPkt_t))		// check the packet size
				{
					bc7215Status.formatPktReady = 1;
					if (bc7215Status.overLap == 0)	// if the format packet is not over writting the data packet
					{
						bitLen = (uint16_t)bufBackRead(datEndPos, 0)<<8 | bufBackRead(datEndPos, 1);
						bc7215Status.dataPktReady = 1;
					}
				}
			}
			else							// if this is the first 0x7a received
			{
				if ((byteCount <= BC7215_BUFFER_SIZE) && !(bufBackRead(lastWritingPos, 2)&0x80))
				// if the buffer is not overflew and 'bit7' of the status byte the is not set (no error)
				{
					temp16 = (uint16_t)bufBackRead(lastWritingPos, 0)<<8 | bufBackRead(lastWritingPos, 1);
					// get the bit count of the data packet

					if ((temp16+7)/8+3 == byteCount)		// if the byte count of received packet is correct
					{
						bitLen = temp16;		// save the bitLen
						bc7215Status.dataPktReady = 1;
						datStartPos = startPos;
						datEndPos = lastWritingPos;
						datCount = byteCount;
					}
				}
			}
			previousData = 0x7a;
			bc7215Status.newPkt = 1;
		}
		else		// incoming data is not 0x7a
		{
			if (bc7215Status.newPkt)			// if it's the start of a new packet
			{
				bc7215Status.newPkt = 0;		// clear new packet indicator
				bc7215Status.overLap = 0;
				byteCount = 0;
				bitLen = 0;
				bc7215Status.dataPktReady = 0;			// new data is coming, clear dataPktReady and formatPktReady flags
				bc7215Status.formatPktReady = 0;
				startPos = lastWritingPos+1;	// save start buffer position of the packet
				if (startPos >= BC7215_BUFFER_SIZE)
				{
					startPos = 0;
				}
			}
			if (data == 0x7b)				// byte-destuffing, ignore 0x7b
			{
				previousData = 0x7b;
				return;
			}
			else
			{
				temp = data;
				if (previousData == 0x7b)
				{
					data &= 0x7f;
				}
				previousData = temp;
			}								// end of byte-destuffing
			// save data to buffer
			lastWritingPos++;
			if (lastWritingPos >= BC7215_BUFFER_SIZE)
			{
				lastWritingPos = 0;
			}
			circularBuffer[lastWritingPos] = data;
			byteCount++;
			if (datCount+byteCount > BC7215_BUFFER_SIZE)
			{
				bc7215Status.overLap = 1;
			}
		}
	}
}


// **************************************
// *** Working mode control functions ***
// **************************************

// set bc7215 to receive(decoder) mode,
// After this function is called, user should wait at least 18ms before any other function calls being made.
void bc7215_set_rx(void)
{
	modSetHighFunction();
	bc7215Status.cmdComplete = 1;
}

// set bc7215 to transmit(encoder) mode,
// After this function is called, user should wait at least 1ms before any other function calls being made .
void bc7215_set_tx(void)
{
	modSetLowFunction();
	bc7215Status.dataPktReady = 0;
	bc7215Status.formatPktReady = 0;
	bc7215Status.newPkt = 1;
	bc7215Status.cmdComplete = 1;
}

// set bc7215 to shutdown mode (only works in tx mode)
void bc7215_set_shutdown(void)
{
	if (readModFunction() == 0)			// if bc7215 is in transmit mode
	{
		uartSendByteFunction(0xf7);
		uartSendByteFunction(0x00);
	}
	bc7215Status.cmdComplete = 0;
}

// set bc7215 receive mode
void bc7215_set_rx_mode(uint8_t mode)
{
	uartSendByteFunction(mode);
}

// ********************************
// *** status inquiry functions ***
// ********************************

// check if there is a data packet received, return non-zero if available
uint8_t bc7215_data_ready(void)
{
	if (readModFunction() != 0)			// if bc7215 is in receive mode
	{
		return bc7215Status.dataPktReady;
	}
	else
	{
		return 0;
	}
}

// check if there is an IR format packet received, return non-zero if available
uint8_t bc7215_format_ready(void)
{
	if (readModFunction() != 0)			// if bc7215 is in receive mode
	{
		return bc7215Status.formatPktReady;
	}
	else
	{
		return 0;
	}
}

// check if bc7215_send_data() or bc7215_set_shutdown_mode() has been completed, return non-zero if completed.
uint8_t bc7215_cmd_completed(void)
{
	if (readModFunction() == 0)			// if bc7215 is in transmit mode
	{
		return bc7215Status.cmdComplete;
	}
	else
	{
		return 1;
	}
}


// ****************************************
// *** Receiving mode related functions ***
// ****************************************

// get the bit length of the received data packet
uint16_t bc7215_get_len(void)
{
	if (bc7215Status.dataPktReady)
	{
		return bitLen;
	}
	else
	{
		return 0;
	}
}

// Get IR payload data when it's available
// copy received IR payload data to "target", return the status/signature byte,
// or "0xff" if data is no longer available (overwritten by new incoming data)
uint8_t bc7215_get_data(bc7215DataVarPkt_t* target)
{	
	uint8_t		i;
	uint8_t		status;

	disUartIntFunction();		// disable UART interrupt
	status = 0xff;
	if (bc7215Status.dataPktReady)
	{
		status = bufBackRead(datEndPos, 2);
		target->bitLen = bitLen;
		for (i=0; i<datCount-3; i++)
		{
			target->data[i] = bufRead(datStartPos, i);
		}
		bc7215Status.dataPktReady = 0;
	}
	enUartIntFunction();			// enable UART interrupt again
	return status;
}


// Get IR format data when it's available
// copy IR format to 'target', return the signature byte,
// or "0xff" if data is no longer available (overwritten by new incomming data)
uint8_t bc7215_get_format(bc7215FormatPkt_t* target)
{
	uint8_t		i;
	uint8_t		signature;
	disUartIntFunction();
	signature = 0xff;
	if (bc7215Status.formatPktReady)
	{
		signature = bufRead(startPos, 0);
		target->signature.byte = signature;
		for (i=0; i<32; i++)
		{
			target->format[i] = bufRead(startPos, i+1);
		}
		bc7215Status.formatPktReady = 0;
	}
	enUartIntFunction();			// enable UART interrupt again
	return signature;
}



// *******************************************
// *** Transmitting mode related functions ***
// *******************************************

// download IR format to bc7215
void bc7215_load_format(bc7215FormatPkt_t* source)
{
	uint8_t	i;
	if (readModFunction() == 0)			// if bc7215 is in transmit mode
	{
		while (readBusyFunction() != 0);		// wait if bc7215 busy is high
		uartSendByteFunction(0xf6);
		while (readBusyFunction() != 0);
		uartSendByteFunction(0x01);
		for (i=0; i<sizeof(bc7215FormatPkt_t); i++)
		{
			byte_stuffing_send(*((uint8_t*)source+i));
		}
	}
}

// Send data via IR
// if data is longer than 16 bytes, this function will only return after all the excessive bytes are sent.
void bc7215_IR_tx(bc7215DataVarPkt_t* source)
{
#if (BC7215_MAX_DATA_BYTE_LENGTH+2)<256

		uint8_t		i;
		uint8_t		bytes;

#else

		uint16_t	i;
		uint16_t	bytes;

#endif
	if ((readModFunction() == 0) && (source->bitLen >= 8))			// check if bc7215 is in trasmitting mode
	{
		bc7215Status.cmdComplete = 0;
		while (readBusyFunction() != 0);		// wait if bc7215 busy is high
		uartSendByteFunction(0xf5);
		while (readBusyFunction() != 0);
		uartSendByteFunction(0x02);
		bytes = sizeof(source->bitLen)+(source->bitLen+7)/8;	// set bytes = total number of data bytes +2
		for (i=0; i<bytes; i++)					// send from 2nd byte of the data packet
		{
			byte_stuffing_send(*((uint8_t*)source+i));
		}
	}
}

static void byte_stuffing_send(uint8_t data)
{
	while(readBusyFunction() != 0);
	if ((data == 0x7a) || (data == 0x7b))
	{
		uartSendByteFunction(0x7b);
		while(readBusyFunction() != 0);
		uartSendByteFunction(data|0x80);
	}
	else
	{
		uartSendByteFunction(data);
	}
}

// set sending IR format to use 56k carrier
void bc7215_set_C56K(bc7215FormatPkt_t* target)
{
	target->signature.bits.c56k = 1;
}

// clear sending IR format to use 37.5k carrier
void bc7215_clr_C56K(bc7215FormatPkt_t* target)
{
	target->signature.bits.c56k = 0;
}

// set sending IR format to use no carrier
void bc7215_set_NOCA(bc7215FormatPkt_t* target)
{
	target->signature.bits.noCA = 1;
}

// clear NOCA bit in format to enable carrier
void bc7215_clr_NOCA(bc7215FormatPkt_t* target)
{
	target->signature.bits.noCA = 0;
}


// *****************************
// *** Buffer read functions ***
// *****************************

// read the (n)th byte backwards from the pos, n range from 0 to BC7215_BUFFER_SIZE-1
#if BC7215_BUFFER_SIZE < 256
	static uint8_t 	bufBackRead(uint8_t pos, uint8_t n)	
#else
	static uint8_t 	bufBackRead(uint16_t pos, uint16_t n)
#endif
{
	if (pos >= n)
	{
		return circularBuffer[pos-n];
	}
	else
	{
		return circularBuffer[BC7215_BUFFER_SIZE+pos-n];
	}
}

// read the (n)th byte start from the pos, n range from 0 to BC7215_BUFFER_SIZE-1
#if BC7215_BUFFER_SIZE < 256
	static uint8_t bufRead(uint8_t pos, uint8_t n)
#else
	static uint8_t bufRead(uint16_t pos, uint16_t n)
#endif
{
	if (pos+n >= BC7215_BUFFER_SIZE)
	{
		return circularBuffer[pos+n-BC7215_BUFFER_SIZE];
	}
	else
	{
		return circularBuffer[pos+n];
	}
}

// calculate CRC-8
#if BC7215_MAX_DATA_BYTE_LENGTH < 256
  uint8_t bc7215_crc8(uint8_t* data, uint8_t len)
  {
	uint8_t i;
#else
  uint8_t bc7215_crc8(uint8_t* data, uint16_t len)
  {
	uint16_t i;
#endif
	uint8_t j;
	uint8_t crc = 0;
    for (i = 0; i < len; ++i)
    {
        crc ^= data[i];
        for ( j = 0; j < 8; ++j)
        {
            if (crc & 0x80)
            {
                crc = (crc << 1) ^ BC7215_CRC8_POLY;
            }
            else
            {
                crc <<= 1;
            }
        }
    }
    return crc;
}





